﻿using System.Collections.ObjectModel;
using System.ComponentModel;

namespace AutocompleteBox.WPF
{
	public class AutocompleteModel : INotifyPropertyChanged
	{
		private string text;
		private string selectedItem;
		private IAutocompleteProvider autocompleteProvider;

		public AutocompleteModel()
		{
			this.Suggestions = new ObservableCollection<string>();
		}

		public event PropertyChangedEventHandler PropertyChanged;

		public string Text
		{
			get
			{
				return this.text;
			}
			set
			{
				if (this.text != value)
				{
					this.text = value;
					this.OnPropertyChanged("Text");

					this.AutocompleteProvider.GetSuggestions(value);
				}
			}
		}

		public string SelectedItem
		{
			get
			{
				return this.selectedItem;
			}
			set
			{
				if (this.selectedItem != value)
				{
					this.selectedItem = value;
					this.OnPropertyChanged("SelectedItem");
				}
			}
		}

		public ObservableCollection<string> Suggestions
		{
			get;
			private set;
		}

		public IAutocompleteProvider AutocompleteProvider
		{
			get
			{
				return this.autocompleteProvider;
			}
			set
			{
				if (this.autocompleteProvider != null)
				{
					this.autocompleteProvider.SuggestionsReceived -= this.OnSuggestionsReceived;
				}
				this.autocompleteProvider = value;
				if (this.autocompleteProvider != null)
				{
					this.autocompleteProvider.SuggestionsReceived += this.OnSuggestionsReceived;
				}
			}
		}

		private void OnPropertyChanged(string propertyName)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}

		private void OnSuggestionsReceived(object sender, SuggestionsReceivedEventArgs e)
		{
			bool emptySelection = string.IsNullOrEmpty(this.Text);

			for (int i = this.Suggestions.Count - 1; i >= 0; i--)
			{
				if (emptySelection || this.Suggestions[i] != this.SelectedItem)
				{
					this.Suggestions.RemoveAt(i);
				}
			}

			foreach (string word in e.Suggestions)
			{
				if (!this.Suggestions.Contains(word))
				{
					this.Suggestions.Insert(0, word);
				}
			}
		}
	}
}