﻿using Telerik.Windows.Controls;
using System.Linq;
using System.Windows;
using System.Collections;
using System.Windows.Controls;
using System.Collections.Specialized;
using AutocompleteBehavior;
using System.Diagnostics;

namespace AutocompleteBox
{
	public class DomainDataSourceAutocompleteProvider : DependencyObject, IAutocompleteProvider
	{
		public DomainDataSource ItemsSource
		{
			get { return (DomainDataSource)GetValue(ItemsSourceProperty); }
			set { SetValue(ItemsSourceProperty, value); }
		}

		public static readonly DependencyProperty ItemsSourceProperty =
			DependencyProperty.Register("ItemsSource", typeof(DomainDataSource), typeof(DomainDataSourceAutocompleteProvider), new PropertyMetadata(OnItemsSourceChanged));

		private static void OnItemsSourceChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
		{
			var autocompleteProvider = d as DomainDataSourceAutocompleteProvider;

			var oldDomainDataSource = e.OldValue as DomainDataSource;
			if (oldDomainDataSource != null)
			{
				(oldDomainDataSource.DataView as INotifyCollectionChanged).CollectionChanged -= autocompleteProvider.DomainDataSourceCollectionChanged;
			}
			var newDomainDataSource = e.NewValue as DomainDataSource;
			if (newDomainDataSource != null)
			{
				(newDomainDataSource.DataView as INotifyCollectionChanged).CollectionChanged += autocompleteProvider.DomainDataSourceCollectionChanged;
			}
		}

		private void DomainDataSourceCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
		{
			if (this.ItemsReceived != null)
			{
				this.ItemsReceived(this, new ItemsReceivedEventArgs(this.ItemsSource.Data));
			}
		}

		public event ItemsReceivedEventHandler ItemsReceived;

		public TextSearchMode TextSearchMode
		{
			get;
			set;
		}

		public string TextPath
		{
			get;
			set;
		}

		private FilterDescriptor FilterDescriptor
		{
			get;
			set;
		}

		public void GetItems(string text)
		{
			this.UpdateFilterDescriptor(text);
		}

		private void UpdateFilterDescriptor(string text)
		{
			var isCaseSensitive = this.TextSearchMode == TextSearchMode.ContainsCaseSensitive || this.TextSearchMode == TextSearchMode.StartsWithCaseSensitive;
			FilterOperator filterOperator;
			if (this.TextSearchMode == TextSearchMode.Contains || this.TextSearchMode == TextSearchMode.ContainsCaseSensitive)
			{
				filterOperator = FilterOperator.Contains;
			}
			else
			{
				filterOperator = FilterOperator.StartsWith;
			}

			if (this.FilterDescriptor == null)
			{
				this.FilterDescriptor = new FilterDescriptor(this.TextPath, filterOperator, text) { IsCaseSensitive = isCaseSensitive };
				this.ItemsSource.FilterDescriptors.Clear();
				this.ItemsSource.FilterDescriptors.Add(this.FilterDescriptor);
			}
			else
			{
				this.FilterDescriptor.Value = text;
			}
		}
	}
}